#' Set curl options.
#'
#' Generally you should only need to use this function to set CURL options
#' directly if there isn't already a helpful wrapper function, like 
#' \code{\link{set_cookies}}, \code{\link{add_headers}} or
#' \code{\link{authenticate}}. 
#'
#' To use this function effectively requires some knowledge of CURL, and 
#' CURL options. A complete set of options can be found at 
#' \url{http://linux.die.net/man/3/curl_easy_setopt}. 
#'
#' Within R, the options have slightly different names: the initial
#' \code{CURLOPT_} is removed, all underscores are converted to periods and 
#' the option is given in lower case.  Thus "CURLOPT_SSLENGINE_DEFAULT"
#' becomes "sslengine.default".  See \code{\link[RCurl]{listCurlOptions}} for
#' a complete list of the R name equivalents.
#'
#' Unlike Curl (and RCurl), all configuration options are per request, not
#' per handle.
#' 
#' @seealso \code{\link{set_config}} to set global config defaults, and
#'  \code{\link{with_config}} to temporarily run code with set options.
#' @family config
#' @family ways to set configuration
#' @param ... named Curl options.
#' @export
config <- function(...) {
  options <- list(...)
  
  known <- c(listCurlOptions(), "signature")
  unknown <- setdiff(names(options), known)
  if (length(unknown) > 0) {
    stop("Unknown RCurl options: ", str_c(unknown, collapse = ", "))
  }
  
  structure(options, class = "config")
}

is.config <- function(x) inherits(x, "config")

# Need to make it more reduce(modifyList)

#' @S3method c config
c.config <- function(...) {
  structure(NextMethod(), class = "config")
}

#' @S3method print config
print.config <- function(x, ...) {
  cat("Config: \n")
  str(unclass(x), give.head = FALSE)
}


default_config <- function() {  
  cert <- system.file("CurlSSL/cacert.pem", package = "RCurl")
  
  c(config(
      followlocation = 1L, 
      maxredirs = 10L, 
      encoding = "gzip",
      cainfo = cert
    ), 
    getOption("httr_config")
  )
}

#' Set (and reset) global httr configuration.
#' 
#' @param config Settings as generated by \code{\link{add_headers}}, 
#'   \code{\link{set_cookies}} or \code{\link{authenticate}}.
#' @param override if \code{TRUE}, ignore existing settings, if \code{FALSE},
#'   combine new config with old.
#' @return invisibility, the old global config.
#' @family ways to set configuration
#' @export
#' @examples
#' GET("http://google.com")
#' set_config(verbose())
#' GET("http://google.com")
#' reset_config()
#' GET("http://google.com")
set_config <- function(config, override = FALSE) {
  stopifnot(is.config(config))
  
  old <- getOption("httr_config") %||% config()
  if (!override) config <- c(old, config)
  options(httr_config = config)
  invisible(old)
}

#' @export
#' @rdname set_config
reset_config <- function() set_config(config(), TRUE)

#' Execute code with configuration set.
#'
#' @family ways to set configuration
#' @inheritParams set_config
#' @param expr code to execute under specified configuration
#' @export
#' @examples
#' with_config(verbose(), {
#'   GET("http://had.co.nz")
#'   GET("http://google.com")
#' })
with_config <- function(config = config(), expr, override = FALSE) {
  stopifnot(is.config(config))
  
  old <- set_config(config, override)
  on.exit(set_config(old, override = TRUE))
  force(expr)
}